@@ -9,7 +9,9 @@ from estimagic.optimization import nlopt_optimizers
 from estimagic.optimization import pounders
 from estimagic.optimization import pygmo_optimizers
 from estimagic.optimization import scipy_optimizers
+from estimagic.optimization import simopt_optimizers
 from estimagic.optimization import tao_optimizers
+from estimagic.optimization.tranquilo import tranquilo
 
 
 MODULES = [
@@ -19,10 +21,12 @@ MODULES = [
     nlopt_optimizers,
     pygmo_optimizers,
     scipy_optimizers,
+    simopt_optimizers,
     tao_optimizers,
     bhhh,
     neldermead,
     pounders,
+    tranquilo,
 ]
 
 ALL_ALGORITHMS = {}
